<?php

namespace Doctrine\Tests\ORM\Functional\Ticket;

use Doctrine\ORM\Events;
use Doctrine\ORM\UnitOfWork;
use Doctrine\Tests\Models\CMS\CmsUser;
use ReflectionClass;

/**
 * @group DDC-3123
 */
class DDC3123Test extends \Doctrine\Tests\OrmFunctionalTestCase
{
    protected function setUp() : void
    {
        $this->useModelSet('cms');
        parent::setUp();
    }

    public function testIssue()
    {
        $test = $this;
        $user = new CmsUser();
        $uow  = $this->_em->getUnitOfWork();

        $user->name     = 'Marco';
        $user->username = 'ocramius';

        $this->_em->persist($user);
        $uow->scheduleExtraUpdate($user, ['name' => 'changed name']);

        $listener = $this->getMockBuilder(\stdClass::class)
                         ->setMethods([Events::postFlush])
                         ->getMock();

        $listener
            ->expects($this->once())
            ->method(Events::postFlush)
            ->will($this->returnCallback(function () use ($uow, $test) {
                $class    = new ReflectionClass(UnitOfWork::class);
                $property = $class->getProperty('extraUpdates');
                $property->setAccessible(true);
                $test->assertEmpty(
                    $property->getValue($uow),
                    'ExtraUpdates are reset before postFlush'
                );
            }));

        $this->_em->getEventManager()->addEventListener(Events::postFlush, $listener);

        $this->_em->flush();
    }
}
